==> timsac_1.3.5.tar.gz: timsac/src/thirmof.f <==
      SUBROUTINE THIRMOF(N,MH,XX,XMEAN,CC,CN,C)
C
      INCLUDE 'timsac_f.h'
C
cc	PROGRAM THIRMO
C     PROGRAM 74.6.1.
C-----------------------------------------------------------------------
C     ** DESIGNED BY H. AKAIKE, THE INSTITUTE OF STATISTICAL MATHEMATICS
C     ** PROGRAMMED BY E. ARAHATA, THE INSTITUTE OF STATISTICAL MATHEMAT
C         TOKYO
C     ** DATE OF THE LATEST REVISION: MARCH 25, 1977
C     ** THIS PROGRAM WAS ORIGINALLY PUBLISHED IN
C        "TIMSAC-74 A TIME SERIES ANALYSIS AND CONTROL PROGRAM PACKAGE(2
C        BY H. AKAIKE, E. ARAHATA AND T. OZAKI, COMPUTER SCIENCE MONOGRA
C        NO.6 MARCH 1976, THE INSTITUTE OF STATISTICAL MATHEMATICS
C-----------------------------------------------------------------------
C     THIS PROGRAM COMPUTES THE THIRD ORDER MOMENTS.
C     THE OUTPUT OF THIS PROGRAM M,MH,CC(I) AND C(I,J) ARE USED AS THE I
C     TO THE PROGRAM BISPEC (BI-SPECTRUM COMPUTATION).
C     THIS PROGRAM REQUIRES THE FOLLOWING INPUTS;
C     N: DATA LENGTH
C     MH: MAXIMUM LAG
C     DFORM: DATA FORMAT STATEMENT INFORMATION
C     X(I),I=1,N: ORIGINAL DATA
C
cc      !DEC$ ATTRIBUTES DLLEXPORT :: THIRMOF
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      REAL*4 X,XMEAN
cc      DIMENSION X(2000),X1(2000),CC(51),C(51,51)
cc      DIMENSION CN(51),ITAB(7),CPR(7),IIND(7)
cc      DIMENSION DFORM(20)
cxx      DIMENSION X(N),X1(N),CC(MH+1),CN(MH+1),C(MH+1,MH+1)
cxx      DIMENSION XX(N)
      INTEGER :: N, MH
      REAL(8) :: XX(N), XMEAN, CC(MH+1), CN(MH+1), C(MH+1,MH+1)
      REAL(8) :: X(N), X1(N), CST0, CST1, AN, BN, SUM, T1, T2, CX0
cc	DATA IIND / 7*4HJ=   /
C
C     INPUT / OUTPUT DATA FILE OPEN
cc      CHARACTER(100) DFNAM
cc      CALL SETWND
cc      DFNAM='thirmo.out'
cc      CALL FLOPN3(DFNAM,NFL)
cc      IF (NFL.EQ.0) GO TO 999
C
cc      READ(5,1) N,MH
      L1=MH+1
C     INPUT FORMAT SPECIFICATION
cc      READ(5,4) (DFORM(I),I=1,20)
cc    4 FORMAT(20A4)
C     ORIGINAL DATA INPUT AND PRINT OUT
cc      READ(5,DFORM) (X(I),I=1,N)
      DO 9 I=1,N
         X(I)=XX(I)
    9 CONTINUE
cc      WRITE(6,49)
cc      WRITE(6,50)
cc      WRITE(6,51)
cc      WRITE(6,52) N,MH
cc      WRITE(6,53)
cc      WRITE(6,55) ((X(I),I),I=1,N)
C
C     MEAN DELETION
      CST0=0.0D-00
      CST1 =1.0D-00
      AN=N
      BN =CST1/AN
      SUM =CST0
      DO 10 I=1,N
cxx   10 SUM=SUM+X(I)
      SUM=SUM+X(I)
   10 CONTINUE
      XMEAN=BN*SUM
      DO 11 I=1,N
cxx   11 X(I)=X(I)-XMEAN
      X(I)=X(I)-XMEAN
   11 CONTINUE
      DO 20 J=1,L1
      KL1=N-J+1
      T1=CST0
      DO 21 K=1,KL1
      KJ=K+J-1
cc      X1(K) =DBLE(X(KJ))*DBLE(X(K))
      X1(K) =X(KJ)*X(K)
cxx   21 T1=T1+X1(K)
      T1=T1+X1(K)
   21 CONTINUE
      CC(J)=BN*T1
      DO 22 I=J,L1
      KL2=N-I+1
      T2 =CST0
      DO 23 K=1,KL2
      KI=K+I-1
cxx   23 T2=T2+X(KI)*X1(K)
      T2=T2+X(KI)*X1(K)
   23 CONTINUE
      C(I,J)=BN*T2
   22 CONTINUE
   20 CONTINUE
C     NORMALIZATION
      CX0=CC(1)
C     COMMON SUBROUTINE CALL
      CALL CORNOM(CC,CN,L1,CX0,CX0)
C     AUTO COVARIANCE PRINT OUT
cc      WRITE(6,1162)
cc      WRITE(6,162) N,MH,XMEAN
cc      WRITE(6,163)
cc      DO 360 I=1,L1
cc      IM1=I-1
cc  360 WRITE(6,155) IM1,CC(I),CN(I)
C     C(I,J) PRINT OUT
cc      WRITE(6,180)
cc      DO 300 I=1,L1
cc      JG=-1
cc      JS=0
cc      JC=0
cc  319 JC=JC+1
cc      IF  (JC.GT.I) GO TO 305
cc      JS=JS+1
cc      IF  (JS.LE.7) GO TO 310
cc      JS=7
cc  316 WRITE(6,190) (IIND(J),ITAB(J),J=1,JS)
cc  316 CONTINUE
cc      JG=JG+1
cc      IF  (JG) 317,318,317
cc  317 WRITE(6,192) (CPR(J),J=1,JS)
cc  317 CONTINUE
cc      GO TO 315
cc  318 IM1=I-1
cc      WRITE(6,191) IM1,(CPR(J),J=1,JS)
cc  315 IF  (JC.EQ.54321) GO TO 300
cc      JS=1
cc  310 ITAB(JS)=JC-1
cc      CPR(JS)=C(I,JC)
cc      GO TO 319
cc  305 JC=54321
cc      GO TO 316
cc  300 CONTINUE
C     AUTO COVARIANCE PUNCH OUT
cc      WRITE(7,1) N,MH
cc      WRITE(7,2) (CC(I),I=1,L1)
C     C(I,J) PUNCH OUT
cc      DO 320 I=1,L1
cc  320 WRITE(7,2) (C(I,J),J=1,I)
cc      CALL FLCLS3(NFL)
cc  999 CONTINUE
      RETURN
cxx    1 FORMAT(16I5)
cxx    2 FORMAT(4D20.10)
cxx   49 FORMAT(1H ,'PROGRAM 74.6.1. THIRMO')
cxx   50 FORMAT(1H ,'THIRD ORDER MOMENTS COMPUTATION')
cxx   51 FORMAT(/1H ,'INITIAL CONDITION')
cxx   52 FORMAT(/1H ,'N=',I5,5X,'MH=',I5)
cxx   53 FORMAT(/1H ,'ORIGINAL DATA')
cxx   55 FORMAT(5(F13.4,' (',I5,'), '))
cxx 1162 FORMAT(///1H ,'AUTOCOVARIANCE')
cc  162 FORMAT(1H ,'N=',I5,5X,'MH=',I5,5X,'MEAN=',E15.5)
cxx  162 FORMAT(/1H ,'N=',I5,5X,'MH=',I5,5X,'MEAN=',D15.5)
cxx  163 FORMAT(/1H ,4X,'I',6X,'CC(I)=E(X(T+I)X(T))',6X,'NORMALIZED')
cxx  180 FORMAT(///1H ,'C(I,J)=E(X(T+I)X(T+J)X(T))')
cxx  181 FORMAT(1H ,2I5,D17.5)
cxx  155 FORMAT(1H ,I5,D17.5,8X,D17.5)
cxx  190 FORMAT(/1H ,7X,7(6X,A2,I5,4X))
cxx  191 FORMAT(1H ,'I=',I5,7D17.5)
cxx  192 FORMAT(1H ,7X,7D17.5)
      END

==> timsac_1.3.8.tar.gz: timsac/src/thirmof.f <==
      SUBROUTINE THIRMOF(N,MH,XX,XMEAN,CC,CN,C)
C
      INCLUDE 'timsac.h'
C
cc	PROGRAM THIRMO
C     PROGRAM 74.6.1.
C-----------------------------------------------------------------------
C     ** DESIGNED BY H. AKAIKE, THE INSTITUTE OF STATISTICAL MATHEMATICS
C     ** PROGRAMMED BY E. ARAHATA, THE INSTITUTE OF STATISTICAL MATHEMAT
C         TOKYO
C     ** DATE OF THE LATEST REVISION: MARCH 25, 1977
C     ** THIS PROGRAM WAS ORIGINALLY PUBLISHED IN
C        "TIMSAC-74 A TIME SERIES ANALYSIS AND CONTROL PROGRAM PACKAGE(2
C        BY H. AKAIKE, E. ARAHATA AND T. OZAKI, COMPUTER SCIENCE MONOGRA
C        NO.6 MARCH 1976, THE INSTITUTE OF STATISTICAL MATHEMATICS
C-----------------------------------------------------------------------
C     THIS PROGRAM COMPUTES THE THIRD ORDER MOMENTS.
C     THE OUTPUT OF THIS PROGRAM M,MH,CC(I) AND C(I,J) ARE USED AS THE I
C     TO THE PROGRAM BISPEC (BI-SPECTRUM COMPUTATION).
C     THIS PROGRAM REQUIRES THE FOLLOWING INPUTS;
C     N: DATA LENGTH
C     MH: MAXIMUM LAG
C     DFORM: DATA FORMAT STATEMENT INFORMATION
C     X(I),I=1,N: ORIGINAL DATA
C
cxx      IMPLICIT REAL*8(A-H,O-Z)
cc      REAL*4 X,XMEAN
cc      DIMENSION X(2000),X1(2000),CC(51),C(51,51)
cc      DIMENSION CN(51),ITAB(7),CPR(7),IIND(7)
cc      DIMENSION DFORM(20)
cxx      DIMENSION X(N),X1(N),CC(MH+1),CN(MH+1),C(MH+1,MH+1)
cxx      DIMENSION XX(N)
      INTEGER N, MH
      DOUBLE PRECISION XX(N), XMEAN, CC(MH+1), CN(MH+1), C(MH+1,MH+1)
c local
      DOUBLE PRECISION X(N), X1(N), CST0, CST1, AN, BN, SUM, T1, T2, CX0
cc	DATA IIND / 7*4HJ=   /
C
C     INPUT / OUTPUT DATA FILE OPEN
cc      CHARACTER(100) DFNAM
cc      CALL SETWND
cc      DFNAM='thirmo.out'
cc      CALL FLOPN3(DFNAM,NFL)
cc      IF (NFL.EQ.0) GO TO 999
C
cc      READ(5,1) N,MH
      L1=MH+1
C     INPUT FORMAT SPECIFICATION
cc      READ(5,4) (DFORM(I),I=1,20)
cc    4 FORMAT(20A4)
C     ORIGINAL DATA INPUT AND PRINT OUT
cc      READ(5,DFORM) (X(I),I=1,N)
      DO 9 I=1,N
         X(I)=XX(I)
    9 CONTINUE
cc      WRITE(6,49)
cc      WRITE(6,50)
cc      WRITE(6,51)
cc      WRITE(6,52) N,MH
cc      WRITE(6,53)
cc      WRITE(6,55) ((X(I),I),I=1,N)
C
C     MEAN DELETION
      CST0=0.0D-00
      CST1 =1.0D-00
      AN=N
      BN =CST1/AN
      SUM =CST0
      DO 10 I=1,N
cxx   10 SUM=SUM+X(I)
      SUM=SUM+X(I)
   10 CONTINUE
      XMEAN=BN*SUM
      DO 11 I=1,N
cxx   11 X(I)=X(I)-XMEAN
      X(I)=X(I)-XMEAN
   11 CONTINUE
      DO 20 J=1,L1
      KL1=N-J+1
      T1=CST0
      DO 21 K=1,KL1
      KJ=K+J-1
cc      X1(K) =DBLE(X(KJ))*DBLE(X(K))
      X1(K) =X(KJ)*X(K)
cxx   21 T1=T1+X1(K)
      T1=T1+X1(K)
   21 CONTINUE
      CC(J)=BN*T1
      DO 22 I=J,L1
      KL2=N-I+1
      T2 =CST0
      DO 23 K=1,KL2
      KI=K+I-1
cxx   23 T2=T2+X(KI)*X1(K)
      T2=T2+X(KI)*X1(K)
   23 CONTINUE
      C(I,J)=BN*T2
   22 CONTINUE
   20 CONTINUE
C     NORMALIZATION
      CX0=CC(1)
C     COMMON SUBROUTINE CALL
      CALL CORNOM(CC,CN,L1,CX0,CX0)
C     AUTO COVARIANCE PRINT OUT
cc      WRITE(6,1162)
cc      WRITE(6,162) N,MH,XMEAN
cc      WRITE(6,163)
cc      DO 360 I=1,L1
cc      IM1=I-1
cc  360 WRITE(6,155) IM1,CC(I),CN(I)
C     C(I,J) PRINT OUT
cc      WRITE(6,180)
cc      DO 300 I=1,L1
cc      JG=-1
cc      JS=0
cc      JC=0
cc  319 JC=JC+1
cc      IF  (JC.GT.I) GO TO 305
cc      JS=JS+1
cc      IF  (JS.LE.7) GO TO 310
cc      JS=7
cc  316 WRITE(6,190) (IIND(J),ITAB(J),J=1,JS)
cc  316 CONTINUE
cc      JG=JG+1
cc      IF  (JG) 317,318,317
cc  317 WRITE(6,192) (CPR(J),J=1,JS)
cc  317 CONTINUE
cc      GO TO 315
cc  318 IM1=I-1
cc      WRITE(6,191) IM1,(CPR(J),J=1,JS)
cc  315 IF  (JC.EQ.54321) GO TO 300
cc      JS=1
cc  310 ITAB(JS)=JC-1
cc      CPR(JS)=C(I,JC)
cc      GO TO 319
cc  305 JC=54321
cc      GO TO 316
cc  300 CONTINUE
C     AUTO COVARIANCE PUNCH OUT
cc      WRITE(7,1) N,MH
cc      WRITE(7,2) (CC(I),I=1,L1)
C     C(I,J) PUNCH OUT
cc      DO 320 I=1,L1
cc  320 WRITE(7,2) (C(I,J),J=1,I)
cc      CALL FLCLS3(NFL)
cc  999 CONTINUE
      RETURN
cxx    1 FORMAT(16I5)
cxx    2 FORMAT(4D20.10)
cxx   49 FORMAT(1H ,'PROGRAM 74.6.1. THIRMO')
cxx   50 FORMAT(1H ,'THIRD ORDER MOMENTS COMPUTATION')
cxx   51 FORMAT(/1H ,'INITIAL CONDITION')
cxx   52 FORMAT(/1H ,'N=',I5,5X,'MH=',I5)
cxx   53 FORMAT(/1H ,'ORIGINAL DATA')
cxx   55 FORMAT(5(F13.4,' (',I5,'), '))
cxx 1162 FORMAT(///1H ,'AUTOCOVARIANCE')
cc  162 FORMAT(1H ,'N=',I5,5X,'MH=',I5,5X,'MEAN=',E15.5)
cxx  162 FORMAT(/1H ,'N=',I5,5X,'MH=',I5,5X,'MEAN=',D15.5)
cxx  163 FORMAT(/1H ,4X,'I',6X,'CC(I)=E(X(T+I)X(T))',6X,'NORMALIZED')
cxx  180 FORMAT(///1H ,'C(I,J)=E(X(T+I)X(T+J)X(T))')
cxx  181 FORMAT(1H ,2I5,D17.5)
cxx  155 FORMAT(1H ,I5,D17.5,8X,D17.5)
cxx  190 FORMAT(/1H ,7X,7(6X,A2,I5,4X))
cxx  191 FORMAT(1H ,'I=',I5,7D17.5)
cxx  192 FORMAT(1H ,7X,7D17.5)
      END
